# Artifact for POPL'24 Paper "Commutativity Simplifies Proofs of Parameterized Programs"

This archive contains the benchmark programs used in the POPL'24 paper _"Commutativity Simplifies Proofs of Parameterized Programs"_ by A. Farzan, D. Klumpp and A. Podelski. <https://doi.org/10.1145/3632925>

## Paper

The paper describes how commutativity-based reasoning can be integrated in verification of parameterized concurrent programs to simplify the required proof.
To demonstrate feasibility of the approach, the paper describes the integration of commutativity reasoning into an approach which reduces the existence of an _Ashcroft invariant_ of a certain fixed width to a satisfiability problem of _Constraint Horn Clauses_ (CHC).

A preprint of the paper can be found at <https://arxiv.org/abs/2311.02673>.

## Implementation

The technique has been implemented as part of the [ULTIMATE program analysis framework](https://ultimate-pa.org/).
Our implementation generates the respective CHC encoding and then runs an off-the-shelf CHC solver to determine the outcome.
The source code can be found in [the ULTIMATE repository](https://github.com/ultimate-pa/ultimate) on github.

## Archive

For each benchmark `<X>`, this archive contains the following:

- `<X>/<X>.bpl` : The benchmark program itself, as Boogie program.
- `<X>/ThreadModularVerifier.Settings.epf` : Settings for the verification of the program.
  Among other settings, this file determines the semantics of the program,
  e.g. which threads are started initially, and which kind of specification shall be checked.
- `<X>/<X>.<C>.smt2` : The CHC encodings generated for the programs, for one configuration `<C>` of the three configurations (`NoSleep`, `SymbolicSleep`, `ExplicitSleep`) described and evaluated in the paper.

Note that the `mutex-<N>` benchmark programs are identical to the respective `mutex-<N>-scaled` programs.
The difference between these folders lies in the width of the Ashcroft invariant whose existence is checked.
This width is determined by the file `ThreadModularVerifier.Settings.epf`.
Naturally, the generated CHC encodings for different widths also differ.

The archive also contains this `README.md`,
as well as the file `ThreadModularVerifier.xml`, which describes the toolchain executed by ULTIMATE to realize the approach described in the paper.

## Reproduction

To retrieve the specific version of our tool used for the evaluation in the paper and the creation of this artifact, check out tag `popl24-artifact` in [the ULTIMATE repository](https://github.com/ultimate-pa/ultimate).

A basic invocation of ULTIMATE looks like this:

```
./Ultimate -tc ThreadModularVerifier.xml \
   -s <X>/ThreadModularVerifier.Settings.epf \
   -i <X>/<X>.bpl
```

The following additional arguments were passed to ULTIMATE:

- depending on the configuration (`NoSleep`, `SymbolicSleep`, `ExplicitSleep`):
  + `--icfgtochc.enable.sleep.set.reduction false` for `NoSleep`
  + `--icfgtochc.break.symmetry.of.preference.order false` for `SymbolicSleep`
  + `ExplicitSleep` corresponds to the settings specified in the various `ThreadModularVerifier.Settings.epf` files, and thus does not require additional arguments.
- depending on the CHC solver used as backend (one of `Z3`, `Eldarica` or `golem`):
  + for eldarica:
    ```
    --chcsolver.chc.solver.backend ELDARICA
    --chcsolver.produce.chc.model.if.query.is.sat true
    --chcsolver.produce.derivation.if.query.is.unsat true
    --chcsolver.produce.unsat.core.if.query.is.unsat true
    ```
  + for Z3:
    ```
    --chcsolver.chc.solver.backend Z3
    --chcsolver.produce.chc.model.if.query.is.sat true
    --chcsolver.produce.derivation.if.query.is.unsat false
    --chcsolver.produce.unsat.core.if.query.is.unsat false
    ```
  + for golem:
    ```
    --chcsolver.chc.solver.backend GOLEM
    --chcsolver.produce.chc.model.if.query.is.sat true
    --chcsolver.produce.derivation.if.query.is.unsat false
    --chcsolver.produce.unsat.core.if.query.is.unsat false
    ```

The evaluation used `Z3 version 4.12.0`, `golem v0.4.2` and `Eldarica v2.0.8`.